function get_3Dview(depth, facecolor, varargin)
%GET_3DVIEW renders a 3D view of original image based on color matrix ('facecolor') and depth matrix ('depth').
%
%   GET_3DVIEW(DEPTH, FACECOLOR,'ATTRIBUTE1',VALUE1,'ATTRIBUTE2',VALUE2...)
%   The order of the attribute-value pairs does not matter, as long as an
%   appropriate value follows each attribute tag. The ATTRIBUTE can be any
%   of the following fields:
%
%       'step'          Set the size of grid. Integer, 1 by default.
%
%       'meshline'      Set whether or not show the mesh lines,
%                       'on' or 'off', 'off' by default
%
%       'topleft'       Set the topleft coordinates[row,column] of the range to 
%                       be shown in the 3D view. [1,1] by default
%
%       'bottomright'   Set the bottomright coordinates of the range to be shown 
%                       in the 3D view. 
%                       bottomright coordinates of original image by default
%
%       'brightness'    Set the brightness of 3D view, 1 by default
%
%   
%   NOTES:
%       1, FACECOLOR should be in uint16 or unint32
%       2, Don't set 'step' too small while the resolution of original image is very high.
%          In my test, to set step = 1 for a 4386*2920 image caused the
%          Matlab crash, but step = 2 worked fine for it
%       3, You might need zoom in if you set 'meshline' to 'on'. Or you may
%          see a black view (too many black lines stay together)
%
%   EXAMPLES:
%
%   %Render 3D view for the part of original image([300,400] to [600,800]),
%   %the grid size is 4*4 pixels with mesh lines
%   get_3Dview(depth,I_a,'step',4,'meshline','on','topleft',[300,400], 'bottomright',[600,800]);



options = parse_inputs(varargin{:}); % Parse the input parameters to the structure 'options'
[m, n] = size(depth);

% Set the parameters
if isfield(options, 'step')
    step = options.('step');
else
    step = 1;
end
    
if isfield(options, 'meshline')
    if strcmp(options.('meshline'),'on')
        meshline = 1;
    else
        meshline = 0;
    end
else
    meshline = 0;
end

if isfield(options, 'topleft')
    topleft = options.('topleft');
else
    topleft = [1,1];
end

if isfield(options, 'bottomright')
    bottomright = options.('bottomright');
else
    bottomright = [m,n];
end

if isfield(options, 'brightness')
    brightness = options.('brightness');
else
    brightness = 1;
end

 % Set 3D coordinates
 x = topleft(1):step:bottomright(1);
 y = topleft(2):step:bottomright(2);
 z = double(depth(x,y));
 
 % Set the color of the face of 3D view
 color = double(facecolor(x,y,:)) * brightness;
 
 % Plot the colored parametric surface
 surf(y,x,z,color); 
 if ~meshline
     shading interp;
 end
 view(10,-80);
        

function options = parse_inputs(varargin)
%PARSE_INPUT parses varargin of GET_3DVIEW to the structure 'options'

get3Dview_fields = {
    'step'
    'meshline'
    'topleft'
    'bottomright'
    'brightness'
    };

options = struct;  % Create an empty struct in case nothing was specified.
total_args  = nargin;
current_arg = 1;

while (current_arg <= total_args)
    param = varargin{current_arg};
    idx = find(strcmpi(param, get3Dview_fields));
    options.(get3Dview_fields{idx}) = varargin{current_arg + 1};       
    current_arg = current_arg + 2;
end
